/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.application;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.desktop.view.license.GenericLicenseJDialog;
import java.io.File;
import javax.swing.JDialog;
import javax.swing.JFrame;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class LicenseJDialog extends GenericLicenseJDialog 
{
	protected static final String _configurationBaseFileName = "LicenseJDialog";
	public static final String BASE_RTF_FILE_NAME = "Application.License.rtf";

	/**
	 * Creates new form LicenseJDialog
	 */
	public LicenseJDialog( JFrame parent, BaseApplicationConfigurationInterface appConf,
							boolean hasToAcceptLicense )
	{
		super( parent, appConf, hasToAcceptLicense );
//		initComponents();

		setRtfDocument();
	}

	public LicenseJDialog( JFrame parent, BaseApplicationConfigurationInterface appConf )
	{
		this( parent, appConf, true );
	}

	/**
	 * Creates new form LicenseJDialog
	 */
	public LicenseJDialog( JDialog parent, BaseApplicationConfigurationInterface appConf,
							boolean hasToAcceptLicense )
	{
		super( parent, appConf, hasToAcceptLicense );
//		initComponents();

		setRtfDocument();
	}

	public LicenseJDialog( JDialog parent, BaseApplicationConfigurationInterface appConf )
	{
		this(parent, appConf, true);
	}

	@Override
	public String getBaseConfFileName()
	{
		return( _configurationBaseFileName );
	}
	
	public String getRtfFileName()
	{
		return( BASE_RTF_FILE_NAME );
	}
	
	public String getResourcePathToRtfLicense( String language )
	{
		return( getAppliConf().getInternationalPropertiesPathInJar() + "/" + language + "/" + getRtfFileName() );
	}

	public String getLongFileNameToRtfLicense( String language )
	{
		return( getAppliConf().getDefaultLanguageConfigurationFolder( language ) + File.separator + getRtfFileName() );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(LicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(LicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(LicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(LicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				LicenseJDialog dialog = new LicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
*/
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
