/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.fileencoder.configuration;

import com.frojasg1.applications.common.configuration.ConfigurationParent;
import com.frojasg1.general.exceptions.ConfigurationException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import java.io.IOException;
import java.util.Properties;

/**
 *
 * @author Usuario
 */
public class EncryptingConfiguration extends ConfigurationParent
{
	public static String BASE_FILE_NAME = "EncryptingConfiguration";

	public final static String STR_CONF_SIZE_FROM = "SIZE_FROM";
	public final static String STR_CONF_FILE_ENCODER_TYPE = FileEncoderParameters.STR_FILE_ENCODER_TYPE_TAG;
	public final static String STR_CONF_NUM_BYTES_FILE_SLICE = FileEncoderParameters.STR_NUM_BYTES_FILE_SLICE_TAG;
	public final static String STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER = FileEncoderParameters.STR_SIZE_OF_NUMBERS_SIMPLE_ENCODER_TAG;
	public final static String STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER = FileEncoderParameters.STR_SIZE_OF_NUMBERS_REORDERER_ENCODER_TAG;
	public final static String STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER = FileEncoderParameters.STR_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER_TAG;
	public final static String STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER = FileEncoderParameters.STR_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER_TAG;
	
	public EncryptingConfiguration()
	{
		super();
	}
	
	public EncryptingConfiguration( EncryptingConfiguration other )
	{
		super();

		if( other.get(STR_CONF_SIZE_FROM) != null )
			this.M_setIntParamConfiguration( STR_CONF_SIZE_FROM, other.M_getIntParamConfiguration( STR_CONF_SIZE_FROM ) );
		if( other.get(STR_CONF_FILE_ENCODER_TYPE) != null )
			this.M_setIntParamConfiguration( STR_CONF_FILE_ENCODER_TYPE, other.M_getIntParamConfiguration( STR_CONF_FILE_ENCODER_TYPE ) );
		if( other.get(STR_CONF_NUM_BYTES_FILE_SLICE) != null )
			this.M_setIntParamConfiguration( STR_CONF_NUM_BYTES_FILE_SLICE, other.M_getIntParamConfiguration( STR_CONF_NUM_BYTES_FILE_SLICE ) );
		if( other.get(STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER) != null )
			this.M_setIntParamConfiguration( STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER, other.M_getIntParamConfiguration( STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER ) );
		if( other.get(STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER) != null )
			this.M_setIntParamConfiguration( STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER, other.M_getIntParamConfiguration( STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER ) );
		if( other.get(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER) != null )
			this.M_setIntParamConfiguration( STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER, other.M_getIntParamConfiguration( STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER ) );
		if( other.get(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER) != null )
			this.M_setIntParamConfiguration( STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER, other.M_getIntParamConfiguration( STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER ) );
	}

	protected Properties M_getDefaultProperties( String language )
	{
		return( null );
	}

	protected void M_openConfigurationFromClassPath( String propName ) throws ConfigurationException, IOException
	{
		Properties properties = null;
		properties = cargarPropertiesClassPath( propName );
		a_properties = properties; // only if there is no exception, we change the values.
		
		if( a_defaultProperties == null ) a_defaultProperties = M_getDefaultProperties(a_language);
		
		if( (a_properties == null) && (a_defaultProperties == null) )
			throw( new ConfigurationException( "The configuration file in class path: " + propName + " could not be open" ) );
	}

	public String[] M_getParamListFromEncryptingConfiguration()
	{
		String result[] = null;

		String fileEncoderType = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_FILE_ENCODER_TYPE);
		String numberOfBytesFileSlice = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_NUM_BYTES_FILE_SLICE);
		String sizeOfNumbersSimpleEncoder = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER);
		String sizeOfNumbersReordererEncoder = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER);
		String numberOfBitsPerIterationSimpleEncoder = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER);
		String numberOfBitsPerIterationReordererEncoder = M_getStrParamConfiguration( EncryptingConfiguration.STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER);

		int numberOfParams = 0;
		
		if( fileEncoderType != null ) numberOfParams = numberOfParams + 2;
		if( numberOfBytesFileSlice != null ) numberOfParams = numberOfParams + 2;
		if( sizeOfNumbersSimpleEncoder != null ) numberOfParams = numberOfParams + 2;
		if( sizeOfNumbersReordererEncoder != null ) numberOfParams = numberOfParams + 2;
		if( numberOfBitsPerIterationSimpleEncoder != null ) numberOfParams = numberOfParams + 2;
		if( numberOfBitsPerIterationReordererEncoder != null ) numberOfParams = numberOfParams + 2;

		if( numberOfParams > 0 )
		{
			result = new String[numberOfParams];
			int ii=0;
			if( fileEncoderType != null )
			{
				result[ii] = FileEncoderParameters.STR_FILE_ENCODER_TYPE_TAG;
				result[ii+1] = fileEncoderType;
				ii=ii+2;
			}

			if( numberOfBytesFileSlice != null )
			{
				result[ii] = FileEncoderParameters.STR_NUM_BYTES_FILE_SLICE_TAG;
				result[ii+1] = numberOfBytesFileSlice;
				ii=ii+2;
			}

			if( sizeOfNumbersSimpleEncoder != null )
			{
				result[ii] = FileEncoderParameters.STR_SIZE_OF_NUMBERS_SIMPLE_ENCODER_TAG;
				result[ii+1] = sizeOfNumbersSimpleEncoder;
				ii=ii+2;
			}

			if( sizeOfNumbersReordererEncoder != null )
			{
				result[ii] = FileEncoderParameters.STR_SIZE_OF_NUMBERS_REORDERER_ENCODER_TAG;
				result[ii+1] = sizeOfNumbersReordererEncoder;
				ii=ii+2;
			}

			if( numberOfBitsPerIterationSimpleEncoder != null )
			{
				result[ii] = FileEncoderParameters.STR_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER_TAG;
				result[ii+1] = numberOfBitsPerIterationSimpleEncoder;
				ii=ii+2;
			}

			if( numberOfBitsPerIterationReordererEncoder != null )
			{
				result[ii] = FileEncoderParameters.STR_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER_TAG;
				result[ii+1] = numberOfBitsPerIterationReordererEncoder;
				ii=ii+2;
			}
		}

		return( result );
	}

	public void M_getParamListFromEncryptingConfiguration( FileEncoderParameters fep )
	{
		int fileEncoderType = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_FILE_ENCODER_TYPE);
		int numberOfBytesFileSlice = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_NUM_BYTES_FILE_SLICE);
		int sizeOfNumbersSimpleEncoder = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER);
		int sizeOfNumbersReordererEncoder = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER);
		int numberOfBitsPerIterationSimpleEncoder = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER);
		int numberOfBitsPerIterationReordererEncoder = M_getIntParamConfiguration( EncryptingConfiguration.STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER);

		if( fileEncoderType > 0 ) fep.M_setFileEncoderType( fileEncoderType );
		if( numberOfBytesFileSlice > 0 ) fep.M_setSliceSize( numberOfBytesFileSlice );
		if( sizeOfNumbersSimpleEncoder > 0 ) fep.M_setSizeOfNumbersSimpleEncoder( sizeOfNumbersSimpleEncoder );
		if( sizeOfNumbersReordererEncoder > 0 ) fep.M_setSizeOfNumbersReordererEncoder( sizeOfNumbersReordererEncoder );
		if( numberOfBitsPerIterationSimpleEncoder > 0 ) fep.M_setNumberOfBitsPerIterationSimpleEncoder(numberOfBitsPerIterationSimpleEncoder );
		if( numberOfBitsPerIterationReordererEncoder > 0 ) fep.M_setNumberOfBitsPerIterationReordererEncoder(numberOfBitsPerIterationReordererEncoder );
	}

	public static String getFileName( String folder, int index )
	{
		String result =	folder + sa_dirSeparator +
						BASE_FILE_NAME + String.format( "%02d", index ) + ".properties";
		return( result );
	}

	public int getFileEncoderType()
	{
		return( M_getIntParamConfiguration(STR_CONF_FILE_ENCODER_TYPE) );
	}

	public int getNumBytesSliceSize()
	{
		return( M_getIntParamConfiguration(STR_CONF_NUM_BYTES_FILE_SLICE) );
	}

	public int getSizeOfNumbersSimpleEncoder()
	{
		return( M_getIntParamConfiguration(STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER) );
	}

	public int getSizeOfNumbersReordererEncoder()
	{
		return( M_getIntParamConfiguration(STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER) );
	}

	public int getNumberOfBitsPerIterationSimpleEncoder()
	{
		return( M_getIntParamConfiguration(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER) );
	}

	public int getNumberOfBitsPerIterationReordererEncoder()
	{
		return( M_getIntParamConfiguration(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER) );
	}

	public void setFileEncoderType( int value )
	{
		M_setIntParamConfiguration(STR_CONF_FILE_ENCODER_TYPE, value );
	}

	public void setNumBytesSliceSize( int value )
	{
		M_setIntParamConfiguration(STR_CONF_NUM_BYTES_FILE_SLICE, value );
	}

	public void setSizeOfNumbersSimpleEncoder( int value )
	{
		M_setIntParamConfiguration(STR_CONF_SIZE_OF_NUMBERS_SIMPLE_ENCODER, value );
	}

	public void setSizeOfNumbersReordererEncoder( int value )
	{
		M_setIntParamConfiguration(STR_CONF_SIZE_OF_NUMBERS_REORDERER_ENCODER, value );
	}

	public void setNumberOfBitsPerIterationSimpleEncoder( int value )
	{
		M_setIntParamConfiguration(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_SIMPLE_ENCODER, value );
	}

	public void setNumberOfBitsPerIterationReordererEncoder( int value )
	{
		M_setIntParamConfiguration(STR_CONF_NUMBER_OF_BITS_PER_ITERATION_REORDERER_ENCODER, value );
	}
}
