/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.fileencoder.fileencodertypes;

import com.hotmail.frojasg1.applications.fileencoder.FileEncoderException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import com.frojasg1.general.progress.UpdatingProgress;
import com.hotmail.frojasg1.encrypting.encoderdecoder.EncoderDecoder;
import com.hotmail.frojasg1.encrypting.encoderdecoder.ReordererEncoder;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGenerator;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGeneratorException;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.chaoticgenerator.ChaoticGenerator_BigLong_depretated;

/**
 *
 * @author Usuario
 */
@Deprecated
public class FileEncoderType_2 extends FileEncoderType_1
{
	public FileEncoderType_2( UpdatingProgress up )
	{
		super( up );
	}

	@Override
	public EncoderDecoder M_newEncoderDecoder( FileEncoderParameters fep2 ) throws FileEncoderException
	{
		EncoderDecoder result = null;
		
		M_checkSizeOfNumbers( fep2 );
		
		FileEncoderParameters fep = ( (fep2==null) ? a_fep : fep2 );
		int sizeOfNumbersSimpleEncoder = fep.M_getSizeOfNumbersSimpleEncoder();
		int sizeOfNumbersReordererEncoder = fep.M_getSizeOfNumbersReordererEncoder();
		int numberOfBitsPerIterationSimpleEncoder = fep.M_getNumberOfBitsPerIterationSimpleEncoder();
		int numberOfBitsPerIterationReordererEncoder = fep.M_getNumberOfBitsPerIterationReordererEncoder();
		
		try
		{
			PseudoRandomGenerator prg_se = null;
			PseudoRandomGenerator prg_sr = null;
				
			prg_se = new ChaoticGenerator_BigLong_depretated( sizeOfNumbersSimpleEncoder, numberOfBitsPerIterationSimpleEncoder );
			prg_sr = new ChaoticGenerator_BigLong_depretated( sizeOfNumbersReordererEncoder, numberOfBitsPerIterationReordererEncoder );
				
			result = new ReordererEncoder( prg_se, prg_sr );
		}
		catch( PseudoRandomGeneratorException ex )
		{
			ex.printStackTrace();
			throw( new FileEncoderException( ex.getMessage() ) );
		}
		
		return( result );
	}

	@Override
	protected EncoderDecoder M_newEncoderDecoder_forShortKey_1( int numberSize, int passwordLength ) throws FileEncoderException
	{
		EncoderDecoder result = null;

		try
		{
			PseudoRandomGenerator prg_se = null;
			PseudoRandomGenerator prg_sr = null;
				
			prg_se = new ChaoticGenerator_BigLong_depretated( numberSize, 1 );
			if( prg_se.M_getRecommendedNumberOfBytesToInitializeKey() >= passwordLength )
			{
				prg_sr = new ChaoticGenerator_BigLong_depretated( numberSize, 1 );
				result = new ReordererEncoder( prg_se, prg_sr );
			}
		}
		catch( PseudoRandomGeneratorException ex )
		{
			ex.printStackTrace();
			throw( new FileEncoderException( ex.getMessage() ) );
		}
		
		return( result );
	}

	public int M_getFileEncoderType()
	{
		return( FileEncoderParameters.INT_FILE_ENCODER_TYPE_2 );
	}
}
