/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.fileencoder.fileencodertypes;

import com.frojasg1.general.GeneralException;
import com.frojasg1.general.HexadecimalFunctions;
import com.frojasg1.general.progress.CancellationException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderException;
import com.hotmail.frojasg1.applications.fileencoder.FileEncoderParameters;
import com.hotmail.frojasg1.encrypting.encoderdecoder.EncoderDecoder;
import com.hotmail.frojasg1.encrypting.encoderdecoder.ReordererEncoder;
import com.frojasg1.general.progress.UpdatingProgress;
import com.hotmail.frojasg1.encrypting.encoderdecoder.SimpleEncoder;
import com.hotmail.frojasg1.encrypting.encoderdecoder.SimpleReorderer;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGenerator;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.PseudoRandomGeneratorException;
import com.hotmail.frojasg1.encrypting.pseudorandomgenerator.chaoticgenerator.ChaoticGenerator_BigLong;

/**
 *
 * @author axpe
 */
public class FileEncoderType_6 extends FileEncoderType_5
{
	public FileEncoderType_6( UpdatingProgress up )
	{
		super( up );
	}

	@Override
	public EncoderDecoder M_newEncoderDecoder( FileEncoderParameters fep2 ) throws FileEncoderException
	{
		EncoderDecoder result = null;
		
		M_checkSizeOfNumbers( fep2 );
		
		FileEncoderParameters fep = ( (fep2==null) ? a_fep : fep2 );
		int sizeOfNumbersSimpleEncoder = fep.M_getSizeOfNumbersSimpleEncoder();
//		int sizeOfNumbersReordererEncoder = fep.M_getSizeOfNumbersReordererEncoder();
		int numberOfBitsPerIterationSimpleEncoder = fep.M_getNumberOfBitsPerIterationSimpleEncoder();
//		int numberOfBitsPerIterationReordererEncoder = fep.M_getNumberOfBitsPerIterationReordererEncoder();
		
		try
		{
			PseudoRandomGenerator prg_se = null;
			PseudoRandomGenerator prg_sr = null;
				
			prg_se = new ChaoticGenerator_BigLong( sizeOfNumbersSimpleEncoder, numberOfBitsPerIterationSimpleEncoder );
//			prg_sr = new ChaoticGenerator_BigLong( sizeOfNumbersReordererEncoder, numberOfBitsPerIterationReordererEncoder );

//			result = new ReordererEncoder( prg_se, prg_sr );
			result = new SimpleEncoder( prg_se );
		}
		catch( PseudoRandomGeneratorException ex )
		{
			ex.printStackTrace();
			throw( new FileEncoderException( ex.getMessage() ) );
		}

		return( result );
	}

	protected byte[] M_encodeHeader_FET_5( byte[] encodedKey, byte[] signature,
									long fileLength ) throws FileEncoderException
	{
		byte[] header_head_buffer = a_fep.M_getHeaderHead().getBytes();

		byte[] variable_part_buffer = new byte[3];
		variable_part_buffer[0] = ( new Integer( a_fep.M_getFileEncoderType() ) ).byteValue();
		variable_part_buffer[1] = getByteValueOfSize( a_fep.M_getSizeOfNumbersSimpleEncoder() );
//		variable_part_buffer[2] = ( new Integer( a_fep.M_getSizeOfNumbersReordererEncoder() ) ).byteValue();
		variable_part_buffer[2] = ( new Integer( a_fep.M_getNumberOfBitsPerIterationSimpleEncoder() ) ).byteValue();
//		variable_part_buffer[4] = ( new Integer( a_fep.M_getNumberOfBitsPerIterationReordererEncoder() ) ).byteValue();

		byte[] sliceSize_buffer = HexadecimalFunctions.M_getBufferFromInteger(a_fep.M_getNumberOfBytesOfFileSlice() );

		byte[] lengthOfFile_buffer = HexadecimalFunctions.M_getBufferFromLong( fileLength );

		byte[] result = new byte[ header_head_buffer.length + variable_part_buffer.length +
									sliceSize_buffer.length + lengthOfFile_buffer.length +
									encodedKey.length + signature.length ];

		System.arraycopy(header_head_buffer, 0, result, 0, header_head_buffer.length );
		System.arraycopy(variable_part_buffer, 0,
							result, header_head_buffer.length, variable_part_buffer.length );
		System.arraycopy(sliceSize_buffer, 0, result,
							header_head_buffer.length + variable_part_buffer.length,
							sliceSize_buffer.length );
		System.arraycopy(lengthOfFile_buffer, 0, result,
							header_head_buffer.length + variable_part_buffer.length +
							sliceSize_buffer.length,
							lengthOfFile_buffer.length );
		System.arraycopy(encodedKey, 0,
							result, header_head_buffer.length + variable_part_buffer.length +
							sliceSize_buffer.length + lengthOfFile_buffer.length,
							encodedKey.length );
		System.arraycopy( signature, 0, result,	header_head_buffer.length +
									variable_part_buffer.length +
									sliceSize_buffer.length + lengthOfFile_buffer.length +
									encodedKey.length,
									signature.length );

		return( result );
	}

	protected void M_updateParametersWithHeader( byte[] header, String fileName ) throws FileEncoderException
	{
		int lenHead = a_fep.M_getHeaderHead().getBytes().length;
		
		byte[] fileHead_buffer = new byte[lenHead];
		byte[] variable_part_buffer = new byte[3];
		byte[] sliceSize_buffer = new byte[4];
		byte[] lengthOfFile_buffer = new byte[8];

		System.arraycopy(header, 0, fileHead_buffer, 0, lenHead );
		System.arraycopy(header, lenHead, variable_part_buffer, 0, 3 );
		System.arraycopy(header, lenHead + variable_part_buffer.length, sliceSize_buffer, 0, 4 );
		System.arraycopy(header, lenHead + variable_part_buffer.length + sliceSize_buffer.length,
							lengthOfFile_buffer, 0, 8);

		String fileHead = new String( fileHead_buffer );
		if( ! fileHead.equals( a_fep.M_getHeaderHead() ) )	throw( new FileEncoderException( "HeaderHead unexpected in file " + fileName ) );
		
		int sliceSize = -1;
		try
		{
			sliceSize = HexadecimalFunctions.M_getIntegerFromBuffer(sliceSize_buffer);
		}
		catch( GeneralException ge )
		{
			ge.printStackTrace();
			throw( new FileEncoderException( "Error getting sliceSize from buffer in file: " + fileName ) );
		}

		try
		{
			_decodedFileLength = HexadecimalFunctions.M_getLongFromBuffer(lengthOfFile_buffer);
		}
		catch( GeneralException ge )
		{
			ge.printStackTrace();
			throw( new FileEncoderException( "Error getting size of file from buffer in file: " + fileName ) );
		}

		int fileEncoderType = variable_part_buffer[0];
		byte sizeOfNumbersSimpleEncoder = variable_part_buffer[1];
//		int sizeOfNumbersReordererEncoder = variable_part_buffer[2];
		int numberOfBitsPerIterationSimpleEncoder = variable_part_buffer[2];
//		int numberOfBitsPerIterationReordererEncoder = variable_part_buffer[4];

		if( fileEncoderType != M_getFileEncoderType() )	throw( new FileEncoderException( "FileEncoderType unexpected in file " + fileName ) );

		a_fep.M_setFileEncoderType(fileEncoderType);
		a_fep.M_setSliceSize(sliceSize);
		a_fep.M_setSizeOfNumbersSimpleEncoder( getUnsignedValueForSize( sizeOfNumbersSimpleEncoder ) );
//		a_fep.M_setSizeOfNumbersReordererEncoder(sizeOfNumbersReordererEncoder);
		a_fep.M_setNumberOfBitsPerIterationSimpleEncoder(numberOfBitsPerIterationSimpleEncoder);
//		a_fep.M_setNumberOfBitsPerIterationReordererEncoder(numberOfBitsPerIterationReordererEncoder);
	}


	public int M_getFileEncoderType()
	{
		return( FileEncoderParameters.INT_FILE_ENCODER_TYPE_6 );
	}

	protected int[] getRandomOrder( int sliceSize, long fileLength )
			throws FileEncoderException, PseudoRandomGeneratorException, CancellationException
	{
		int[] randomOrder = null;

		SimpleReorderer enc = null;

		int sizeOfNumbersSimpleEncoder = a_fep.M_getSizeOfNumbersSimpleEncoder();
//		int sizeOfNumbersReordererEncoder = fep.M_getSizeOfNumbersReordererEncoder();
		int numberOfBitsPerIterationSimpleEncoder = a_fep.M_getNumberOfBitsPerIterationSimpleEncoder();
//		int numberOfBitsPerIterationReordererEncoder = fep.M_getNumberOfBitsPerIterationReordererEncoder();
		
		try
		{
			PseudoRandomGenerator prg_se = null;
			PseudoRandomGenerator prg_sr = null;
				
			prg_se = new ChaoticGenerator_BigLong( sizeOfNumbersSimpleEncoder, numberOfBitsPerIterationSimpleEncoder );
//			prg_sr = new ChaoticGenerator_BigLong( sizeOfNumbersReordererEncoder, numberOfBitsPerIterationReordererEncoder );

//			result = new ReordererEncoder( prg_se, prg_sr );
			enc = new SimpleReorderer( prg_se );
			enc.M_initializeKey( a_encoder.getKey() );
		}
		catch( PseudoRandomGeneratorException ex )
		{
			ex.printStackTrace();
			throw( new FileEncoderException( ex.getMessage() ) );
		}

		if( ( sliceSize > 0 ) && ( enc != null ) )
		{
			int numberOfSlices = (int) ( ( fileLength - 1 ) / sliceSize + 1 );
			randomOrder = enc.getRandomOrder( numberOfSlices, null);
		}

		return( randomOrder );
	}

	@Override
	public void M_checkSizeOfNumbers( FileEncoderParameters fep ) throws FileEncoderException
	{
		if( ( fep.M_getSizeOfNumbersSimpleEncoder() < 1 ) || (fep.M_getSizeOfNumbersSimpleEncoder() > 256 ) )
			throw( new FileEncoderException( "Bad value for SizeOfNumbersSimpleEncoder. It must be between 1 and 256" ) );
	}

	protected int getOffsetForEncodedKey()
	{
		return( FileEncoderParameters.M_getHeaderHead().getBytes().length + 7 + 8 );
	}

}
