/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.controller.threads;

import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.textcrypt.controller.TextcryptController;

/**
 *
 * @author Usuario
 */
public abstract class ParentThread extends Thread
{
    protected boolean a_hasToStop = false;
    protected boolean a_hasEnded = false;

    private final Object a_lock1 = new Object();
    private final Object a_lock2 = new Object();

    protected boolean a_itWasError = false;
    protected String a_errorString = null;

	protected OperationCancellation a_oc = null;
	
	protected boolean a_isCancellation = false;
	
	protected TextcryptController _controller = null;
	
    public ParentThread( TextcryptController tc, OperationCancellation oc )
    {
        super();

		_controller = tc;
        a_hasToStop = false;
        a_hasEnded = false;
        a_itWasError = false;
        a_errorString = null;
		a_oc = oc;
    }

    @Override
    public abstract void run();

    public boolean M_getItWasError()        { return( a_itWasError );    }
    protected void M_setItWasError( boolean itWasError )
    {
        a_itWasError = itWasError;
    }

    public String M_getErrorString()        { return( a_errorString );    }
    protected void M_setErrorString( String errorString )
    {
        M_setItWasError( true );
        a_errorString = "\nERROR!!!\n" + errorString;
    }

    public boolean M_getHasEnded()
    {
        synchronized( a_lock1 )
        {
            return( a_hasEnded );
        }
    }
    protected synchronized void M_setHasEnded( boolean hasEnded )
    {
        synchronized( a_lock1 )
        {
            a_hasEnded = hasEnded;
        }
    }

    protected boolean M_getHasToStop()
    {
        synchronized( a_lock2 )
        {
            return( a_hasToStop );
        }
    }

    public void M_setHasToStop( boolean hasToStop )
    {
        synchronized( a_lock2 )
        {
            a_hasToStop = hasToStop;
        }
    }

	public boolean M_isCancellation()		{ return( a_isCancellation );	}
	protected void M_setIsCancellation( boolean value )	{ a_isCancellation = value;	}

}
